/**
 * Based on 'Brightness Tracking' by Golan Levin
 */

import processing.video.*;

Capture video;

// Global variables
boolean CONF_CIRCLE = false;
boolean CONF_TRACE = true;
boolean CONF_ELLIPSE = true;

int windowWidth = 640;
int windowHeight = 480;

color compColor = color(104, 14, 3);
int tolerance = 13;
int traceSize = 20;
int count = 0;
int[] traceX = new int[traceSize];
int[] traceY = new int[traceSize];

// Setup
void setup() {
  size(windowWidth, windowHeight); // Change size to 320 x 240 if too slow at 640 x 480

  // Uses the default video input, see the reference if this causes an error
  video = new Capture(this, width, height, 30);
  video.start();
  
  noStroke();
  frameRate(20);
  smooth();
}

// Set new compColor on mouseclick 
void mouseReleased() {
  compColor = get(mouseX, mouseY);
  print("New Color set: " + red(compColor) + ", " + green(compColor) + ", " + blue(compColor));
}

// Key settings
void keyPressed() {
  // t: Increase tolerance
  if(key == '+') 
    tolerance = tolerance + 2;
  // z: Decrease tolerance
  else if(key == '-')
    tolerance = tolerance - 2;
  // e: Toggle ellipse 
  else if(key == 'e')
    CONF_ELLIPSE = (CONF_ELLIPSE) ? false : true;
  // r: Toggle trace
  else if(key == 'r')
    CONF_TRACE = (CONF_TRACE) ? false : true;
  else if (key == 's') {
    save("tracking.png");
  }
}

void draw() {
  if (video.available()) {
    video.read();
    image(video, 0, 0, width, height); // Draw the webcam video onto the screen
    int pointX = 0; // X-coordinate of the brightest video pixel
    int pointY = 0; // Y-coordinate of the brightest video pixel


    boolean objectFound = false;
    int ellipseMinX = windowWidth;
    int ellipseMaxX = 0;
    int ellipseMinY = windowHeight;
    int ellipseMaxY = 0;

    int loops;
    float smallestDiff = 255.0;

    // Search for the brightest pixel: For each row of pixels in the video image and
    // for each pixel in the yth row, compute each pixel's index in the video
    video.loadPixels();
    int index = 0;
    int bestPixel = 0;

    for (int y = 0; y < video.height; y++) {
      for (int x = 0; x < video.width; x++) {
        // Get the color stored in the pixel
        color pixelValue = video.pixels[index];

        // Calculate distance to comparison color
        float diff = abs(red(pixelValue) - red(compColor)) +
          abs(green(pixelValue) - green(compColor)) +
          abs(blue(pixelValue) - blue(compColor));

        // Pixel worth noticing (within tolerated range)
        if(diff < tolerance) {
          objectFound = true;

          // Find point most left
          if(x < ellipseMinX)
            ellipseMinX = x;

          // Find point most right
          if(x > ellipseMaxX)
            ellipseMaxX = x;

          // Find point most bottom
          if(y < ellipseMinY)
            ellipseMinY = y;

          // Find point most top
          if(y > ellipseMaxY)
            ellipseMaxY = y;


          // Find best point
          if(diff < smallestDiff) {
            smallestDiff = diff;
            bestPixel = pixelValue;
            pointY = y;
            pointX = x;
          } 
        }
        index++;
      }
    }

    //print("\nVal: " + bestPixel + " diff: " + smallestDiff);

    // Detect and log object, if within tolerance
    int currentIndex = (count % traceSize);
    if(smallestDiff < tolerance) { 
      count++;

      // Calculate center and ellipse w/h
      int ellWidth = ellipseMaxX-ellipseMinX;
      int ellHeight = ellipseMaxY-ellipseMinY;
      int ellX = ellipseMinX + (int)(ellWidth / 2);
      int ellY = ellipseMinY + (int)(ellHeight / 2);

      // Add point to trace array
      traceX[currentIndex] = ellX; //pointX;
      traceY[currentIndex] = ellY; //pointY;

      // Draw a ellipse circle
      if(CONF_ELLIPSE && objectFound) {
        strokeWeight(3);
        stroke(255, 174, 0);
        //stroke(compColor);
        fill(compColor, 50);
        rectMode(CENTER);
        rect(ellX, ellY, ellWidth, ellHeight);
      }

      // Draw traces
      if(CONF_TRACE) {
        // Stroke settings
        strokeJoin(ROUND);
        strokeWeight(3.0);
        stroke(255, 204, 0);

        // Don't trace uninitialized points
        if(count < traceSize) {
          loops = count+1;
        } 
        else {
          loops = traceSize;
        }
        
        // Trace loop
        for(int i=0; i < loops - 1; i++) {
          int offset = currentIndex+1; //floor(count / traceSize);
          int start = (offset + i) % traceSize;
          int end = (start + 1) % traceSize;

          strokeWeight(i * (ellWidth * ellHeight) / 8000);
          stroke(255, 174, (255 - (10 * i)));
          
          curve(traceX[start], traceY[start], traceX[start], traceY[start], traceX[end], traceY[end], traceX[end], traceY[end]);  
          //line(traceX[start], traceY[start], traceX[end], traceY[end]);  
          //print("\nI: " + i + " Start: " + start + " End : " + end); 
        }
      }
    }

    // Draw a fixed circle
    if(CONF_CIRCLE && (smallestDiff < tolerance)) {
      stroke(compColor);
      fill(255, 174, 0, 128);
      ellipse(pointX, pointY, 50, 50);
    }

 }
}

